/*
  component.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "acn/component.h"

#include <vector>
#include <string>

namespace RDMnet {

/**
 * @brief 6 Broker Protocol
 * 6.1.3 Client
 * A Client is a Component which connects to a Broker. Clients communicate
 * with other Clients which are using the same Client Protocol in the same
 * Scope via their connection to the Broker.
 */

class Component
    : public ACN::Component
{
public:
  Component(UUID::uuid cid = UUID::uuid())
    : ACN::Component(cid)
  {};
  virtual ~Component() {};

private:
  /// @brief 6.2.1 Scopes
  /// Each Component participates in one or more Scopes.
  std::vector<std::string> scopes_;
};

} // namespace RDMnet


