/*
  data.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "acn/pdu.h"
#include "sacn.h"

namespace SACN {
namespace DATA {

/**
 * @brief Table 6-1: E1.31 Data Packet Framing Layer
 */
struct  data_header : ACN::PDU::pdu_header {
  std::string  source_name;
  uint8_t  priority;
  uint16_t sync_address;
  uint8_t  sequence_number;
  uint8_t  options;
  uint16_t universe;

  size_t   streamSize() const override { return 71; }
  void     iStream(ACN::PDU::Stream) override;
  void     oStream(ACN::PDU::Stream) const override;
};


/**
 * @brief 6.2.6 E1.31 Data Packet: Options
 */
enum options_t : uint8_t {
  PREVIEW_DATA          = 0b10000000, //!< Bit 7 = Preview_Data
  STREAM_TERMINATED     = 0b01000000, //!< Bit 6 = Stream_Terminated
  FORCE_SYNCHRONIZATION = 0b00100000, //!< Bit 5 = Force_Synchronization
};


/**
 * @brief The DATA::Pdu class
 */
class Pdu
  : public ACN::PDU::Pdu
{
public:
    Pdu();
    void iStream(ACN::PDU::Stream) override;
};

} // DATA
} // SACN
